@@ -5,10 +5,9 @@ import pytest
 
 from starlette._utils import collapse_excgroups
 from starlette.middleware.wsgi import WSGIMiddleware, build_environ
-from starlette.testclient import TestClient
+from tests.types import TestClientFactory
 
 WSGIResponse = Iterable[bytes]
-TestClientFactory = Callable[..., TestClient]
 StartResponse = Callable[..., Any]
 Environment = Dict[str, Any]
 
