@@ -127,17 +127,13 @@ def make_app_max_parts(max_files: int = 1000, max_fields: int = 1000) -> ASGIApp
     return app
 
 
-def test_multipart_request_data(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_data(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post("/", data={"some": "data"}, files=FORCE_MULTIPART)
     assert response.json() == {"some": "data"}
 
 
-def test_multipart_request_files(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_files(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "test.txt")
     with open(path, "wb") as file:
         file.write(b"<file content>")
@@ -155,9 +151,7 @@ def test_multipart_request_files(
         }
 
 
-def test_multipart_request_files_with_content_type(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_files_with_content_type(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "test.txt")
     with open(path, "wb") as file:
         file.write(b"<file content>")
@@ -175,9 +169,7 @@ def test_multipart_request_files_with_content_type(
         }
 
 
-def test_multipart_request_multiple_files(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_multiple_files(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path1 = os.path.join(tmpdir, "test1.txt")
     with open(path1, "wb") as file:
         file.write(b"<file1 content>")
@@ -188,9 +180,7 @@ def test_multipart_request_multiple_files(
 
     client = test_client_factory(app)
     with open(path1, "rb") as f1, open(path2, "rb") as f2:
-        response = client.post(
-            "/", files={"test1": f1, "test2": ("test2.txt", f2, "text/plain")}
-        )
+        response = client.post("/", files={"test1": f1, "test2": ("test2.txt", f2, "text/plain")})
         assert response.json() == {
             "test1": {
                 "filename": "test1.txt",
@@ -207,9 +197,7 @@ def test_multipart_request_multiple_files(
         }
 
 
-def test_multipart_request_multiple_files_with_headers(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_multiple_files_with_headers(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path1 = os.path.join(tmpdir, "test1.txt")
     with open(path1, "wb") as file:
         file.write(b"<file1 content>")
@@ -281,9 +269,7 @@ def test_multi_items(tmpdir: Path, test_client_factory: TestClientFactory) -> No
         }
 
 
-def test_multipart_request_mixed_files_and_data(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_mixed_files_and_data(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post(
         "/",
@@ -303,11 +289,7 @@ def test_multipart_request_mixed_files_and_data(
             b"value1\r\n"
             b"--a7f7ac8d4e2e437c877bb7b8d7cc549c--\r\n"
         ),
-        headers={
-            "Content-Type": (
-                "multipart/form-data; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c"
-            )
-        },
+        headers={"Content-Type": ("multipart/form-data; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c")},
     )
     assert response.json() == {
         "file": {
@@ -321,26 +303,19 @@ def test_multipart_request_mixed_files_and_data(
     }
 
 
-def test_multipart_request_with_charset_for_filename(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_with_charset_for_filename(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post(
         "/",
         data=(
             # file
             b"--a7f7ac8d4e2e437c877bb7b8d7cc549c\r\n"  # type: ignore
-            b'Content-Disposition: form-data; name="file"; filename="\xe6\x96\x87\xe6\x9b\xb8.txt"\r\n'  # noqa: E501
+            b'Content-Disposition: form-data; name="file"; filename="\xe6\x96\x87\xe6\x9b\xb8.txt"\r\n'
             b"Content-Type: text/plain\r\n\r\n"
             b"<file content>\r\n"
             b"--a7f7ac8d4e2e437c877bb7b8d7cc549c--\r\n"
         ),
-        headers={
-            "Content-Type": (
-                "multipart/form-data; charset=utf-8; "
-                "boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c"
-            )
-        },
+        headers={"Content-Type": ("multipart/form-data; charset=utf-8; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c")},
     )
     assert response.json() == {
         "file": {
@@ -352,25 +327,19 @@ def test_multipart_request_with_charset_for_filename(
     }
 
 
-def test_multipart_request_without_charset_for_filename(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_without_charset_for_filename(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post(
         "/",
         data=(
             # file
             b"--a7f7ac8d4e2e437c877bb7b8d7cc549c\r\n"  # type: ignore
-            b'Content-Disposition: form-data; name="file"; filename="\xe7\x94\xbb\xe5\x83\x8f.jpg"\r\n'  # noqa: E501
+            b'Content-Disposition: form-data; name="file"; filename="\xe7\x94\xbb\xe5\x83\x8f.jpg"\r\n'
             b"Content-Type: image/jpeg\r\n\r\n"
             b"<file content>\r\n"
             b"--a7f7ac8d4e2e437c877bb7b8d7cc549c--\r\n"
         ),
-        headers={
-            "Content-Type": (
-                "multipart/form-data; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c"
-            )
-        },
+        headers={"Content-Type": ("multipart/form-data; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c")},
     )
     assert response.json() == {
         "file": {
@@ -382,9 +351,7 @@ def test_multipart_request_without_charset_for_filename(
     }
 
 
-def test_multipart_request_with_encoded_value(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_request_with_encoded_value(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post(
         "/",
@@ -395,19 +362,12 @@ def test_multipart_request_with_encoded_value(
             b"Transf\xc3\xa9rer\r\n"
             b"--20b303e711c4ab8c443184ac833ab00f--\r\n"
         ),
-        headers={
-            "Content-Type": (
-                "multipart/form-data; charset=utf-8; "
-                "boundary=20b303e711c4ab8c443184ac833ab00f"
-            )
-        },
+        headers={"Content-Type": ("multipart/form-data; charset=utf-8; boundary=20b303e711c4ab8c443184ac833ab00f")},
     )
     assert response.json() == {"value": "Transférer"}
 
 
-def test_urlencoded_request_data(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_urlencoded_request_data(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post("/", data={"some": "data"})
     assert response.json() == {"some": "data"}
@@ -419,37 +379,27 @@ def test_no_request_data(tmpdir: Path, test_client_factory: TestClientFactory) -
     assert response.json() == {}
 
 
-def test_urlencoded_percent_encoding(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_urlencoded_percent_encoding(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post("/", data={"some": "da ta"})
     assert response.json() == {"some": "da ta"}
 
 
-def test_urlencoded_percent_encoding_keys(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_urlencoded_percent_encoding_keys(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.post("/", data={"so me": "data"})
     assert response.json() == {"so me": "data"}
 
 
-def test_urlencoded_multi_field_app_reads_body(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_urlencoded_multi_field_app_reads_body(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app_read_body)
     response = client.post("/", data={"some": "data", "second": "key pair"})
     assert response.json() == {"some": "data", "second": "key pair"}
 
 
-def test_multipart_multi_field_app_reads_body(
-    tmpdir: Path, test_client_factory: TestClientFactory
-) -> None:
+def test_multipart_multi_field_app_reads_body(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app_read_body)
-    response = client.post(
-        "/", data={"some": "data", "second": "key pair"}, files=FORCE_MULTIPART
-    )
+    response = client.post("/", data={"some": "data", "second": "key pair"}, files=FORCE_MULTIPART)
     assert response.json() == {"some": "data", "second": "key pair"}
 
 
@@ -481,7 +431,7 @@ def test_missing_boundary_parameter(
             "/",
             data=(
                 # file
-                b'Content-Disposition: form-data; name="file"; filename="\xe6\x96\x87\xe6\x9b\xb8.txt"\r\n'  # type: ignore # noqa: E501
+                b'Content-Disposition: form-data; name="file"; filename="\xe6\x96\x87\xe6\x9b\xb8.txt"\r\n'  # type: ignore
                 b"Content-Type: text/plain\r\n\r\n"
                 b"<file content>\r\n"
             ),
@@ -513,16 +463,10 @@ def test_missing_name_parameter_on_content_disposition(
                 b'Content-Disposition: form-data; ="field0"\r\n\r\n'
                 b"value0\r\n"
             ),
-            headers={
-                "Content-Type": (
-                    "multipart/form-data; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c"
-                )
-            },
+            headers={"Content-Type": ("multipart/form-data; boundary=a7f7ac8d4e2e437c877bb7b8d7cc549c")},
         )
         assert res.status_code == 400
-        assert (
-            res.text == 'The Content-Disposition header field "name" must be provided.'
-        )
+        assert res.text == 'The Content-Disposition header field "name" must be provided.'
 
 
 @pytest.mark.parametrize(
@@ -540,9 +484,7 @@ def test_too_many_fields_raise(
     client = test_client_factory(app)
     fields = []
     for i in range(1001):
-        fields.append(
-            "--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     with expectation:
         res = client.post(
@@ -569,11 +511,7 @@ def test_too_many_files_raise(
     client = test_client_factory(app)
     fields = []
     for i in range(1001):
-        fields.append(
-            "--B\r\n"
-            f'Content-Disposition: form-data; name="N{i}"; filename="F{i}";\r\n\r\n'
-            "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="N{i}"; filename="F{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     with expectation:
         res = client.post(
@@ -602,11 +540,7 @@ def test_too_many_files_single_field_raise(
     for i in range(1001):
         # This uses the same field name "N" for all files, equivalent to a
         # multifile upload form field
-        fields.append(
-            "--B\r\n"
-            f'Content-Disposition: form-data; name="N"; filename="F{i}";\r\n\r\n'
-            "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="N"; filename="F{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     with expectation:
         res = client.post(
@@ -633,14 +567,8 @@ def test_too_many_files_and_fields_raise(
     client = test_client_factory(app)
     fields = []
     for i in range(1001):
-        fields.append(
-            "--B\r\n"
-            f'Content-Disposition: form-data; name="F{i}"; filename="F{i}";\r\n\r\n'
-            "\r\n"
-        )
-        fields.append(
-            "--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="F{i}"; filename="F{i}";\r\n\r\n' "\r\n")
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     with expectation:
         res = client.post(
@@ -670,9 +598,7 @@ def test_max_fields_is_customizable_low_raises(
     client = test_client_factory(app)
     fields = []
     for i in range(2):
-        fields.append(
-            "--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     with expectation:
         res = client.post(
@@ -702,11 +628,7 @@ def test_max_files_is_customizable_low_raises(
     client = test_client_factory(app)
     fields = []
     for i in range(2):
-        fields.append(
-            "--B\r\n"
-            f'Content-Disposition: form-data; name="F{i}"; filename="F{i}";\r\n\r\n'
-            "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="F{i}"; filename="F{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     with expectation:
         res = client.post(
@@ -718,20 +640,12 @@ def test_max_files_is_customizable_low_raises(
         assert res.text == "Too many files. Maximum number of files is 1."
 
 
-def test_max_fields_is_customizable_high(
-    test_client_factory: TestClientFactory,
-) -> None:
+def test_max_fields_is_customizable_high(test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(make_app_max_parts(max_fields=2000, max_files=2000))
     fields = []
     for i in range(2000):
-        fields.append(
-            "--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n"
-        )
-        fields.append(
-            "--B\r\n"
-            f'Content-Disposition: form-data; name="F{i}"; filename="F{i}";\r\n\r\n'
-            "\r\n"
-        )
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="N{i}";\r\n\r\n' "\r\n")
+        fields.append("--B\r\n" f'Content-Disposition: form-data; name="F{i}"; filename="F{i}";\r\n\r\n' "\r\n")
     data = "".join(fields).encode("utf-8")
     data += b"--B--\r\n"
     res = client.post(
@@ -748,3 +662,40 @@ def test_max_fields_is_customizable_high(
         "content": "",
         "content_type": None,
     }
+
+
+@pytest.mark.parametrize(
+    "app,expectation",
+    [
+        (app, pytest.raises(MultiPartException)),
+        (Starlette(routes=[Mount("/", app=app)]), does_not_raise()),
+    ],
+)
+def test_max_part_size_exceeds_limit(
+    app: ASGIApp,
+    expectation: typing.ContextManager[Exception],
+    test_client_factory: TestClientFactory,
+) -> None:
+    client = test_client_factory(app)
+    boundary = "------------------------4K1ON9fZkj9uCUmqLHRbbR"
+
+    multipart_data = (
+        f"--{boundary}\r\n"
+        f'Content-Disposition: form-data; name="small"\r\n\r\n'
+        "small content\r\n"
+        f"--{boundary}\r\n"
+        f'Content-Disposition: form-data; name="large"\r\n\r\n'
+        + ("x" * 1024 * 1024 + "x")  # 1MB + 1 byte of data
+        + "\r\n"
+        f"--{boundary}--\r\n"
+    ).encode("utf-8")
+
+    headers = {
+        "Content-Type": f"multipart/form-data; boundary={boundary}",
+        "Transfer-Encoding": "chunked",
+    }
+
+    with expectation:
+        response = client.post("/", data=multipart_data, headers=headers)  # type: ignore
+        assert response.status_code == 400
+        assert response.text == "Part exceeded maximum size of 1024KB."
