@@ -2,6 +2,7 @@ import os
 import stat
 import tempfile
 import time
+import typing
 from pathlib import Path
 
 import anyio
@@ -10,13 +11,15 @@ import pytest
 from starlette.applications import Starlette
 from starlette.exceptions import HTTPException
 from starlette.middleware import Middleware
-from starlette.middleware.base import BaseHTTPMiddleware
+from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
 from starlette.requests import Request
+from starlette.responses import Response
 from starlette.routing import Mount
 from starlette.staticfiles import StaticFiles
+from tests.types import TestClientFactory
 
 
-def test_staticfiles(tmpdir, test_client_factory):
+def test_staticfiles(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -28,7 +31,7 @@ def test_staticfiles(tmpdir, test_client_factory):
     assert response.text == "<file content>"
 
 
-def test_staticfiles_with_pathlib(tmp_path: Path, test_client_factory):
+def test_staticfiles_with_pathlib(tmp_path: Path, test_client_factory: TestClientFactory) -> None:
     path = tmp_path / "example.txt"
     with open(path, "w") as file:
         file.write("<file content>")
@@ -40,7 +43,7 @@ def test_staticfiles_with_pathlib(tmp_path: Path, test_client_factory):
     assert response.text == "<file content>"
 
 
-def test_staticfiles_head_with_middleware(tmpdir, test_client_factory):
+def test_staticfiles_head_with_middleware(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     """
     see https://github.com/encode/starlette/pull/935
     """
@@ -48,7 +51,7 @@ def test_staticfiles_head_with_middleware(tmpdir, test_client_factory):
     with open(path, "w") as file:
         file.write("x" * 100)
 
-    async def does_nothing_middleware(request: Request, call_next):
+    async def does_nothing_middleware(request: Request, call_next: RequestResponseEndpoint) -> Response:
         response = await call_next(request)
         return response
 
@@ -62,7 +65,7 @@ def test_staticfiles_head_with_middleware(tmpdir, test_client_factory):
     assert response.headers.get("content-length") == "100"
 
 
-def test_staticfiles_with_package(test_client_factory):
+def test_staticfiles_with_package(test_client_factory: TestClientFactory) -> None:
     app = StaticFiles(packages=["tests"])
     client = test_client_factory(app)
     response = client.get("/example.txt")
@@ -76,7 +79,7 @@ def test_staticfiles_with_package(test_client_factory):
     assert response.text == "123\n"
 
 
-def test_staticfiles_post(tmpdir, test_client_factory):
+def test_staticfiles_post(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -90,7 +93,7 @@ def test_staticfiles_post(tmpdir, test_client_factory):
     assert response.text == "Method Not Allowed"
 
 
-def test_staticfiles_with_directory_returns_404(tmpdir, test_client_factory):
+def test_staticfiles_with_directory_returns_404(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -104,7 +107,7 @@ def test_staticfiles_with_directory_returns_404(tmpdir, test_client_factory):
     assert response.text == "Not Found"
 
 
-def test_staticfiles_with_missing_file_returns_404(tmpdir, test_client_factory):
+def test_staticfiles_with_missing_file_returns_404(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -118,14 +121,14 @@ def test_staticfiles_with_missing_file_returns_404(tmpdir, test_client_factory):
     assert response.text == "Not Found"
 
 
-def test_staticfiles_instantiated_with_missing_directory(tmpdir):
+def test_staticfiles_instantiated_with_missing_directory(tmpdir: Path) -> None:
     with pytest.raises(RuntimeError) as exc_info:
         path = os.path.join(tmpdir, "no_such_directory")
         StaticFiles(directory=path)
     assert "does not exist" in str(exc_info.value)
 
 
-def test_staticfiles_configured_with_missing_directory(tmpdir, test_client_factory):
+def test_staticfiles_configured_with_missing_directory(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "no_such_directory")
     app = StaticFiles(directory=path, check_dir=False)
     client = test_client_factory(app)
@@ -135,8 +138,8 @@ def test_staticfiles_configured_with_missing_directory(tmpdir, test_client_facto
 
 
 def test_staticfiles_configured_with_file_instead_of_directory(
-    tmpdir, test_client_factory
-):
+    tmpdir: Path, test_client_factory: TestClientFactory
+) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -148,7 +151,7 @@ def test_staticfiles_configured_with_file_instead_of_directory(
     assert "is not a directory" in str(exc_info.value)
 
 
-def test_staticfiles_config_check_occurs_only_once(tmpdir, test_client_factory):
+def test_staticfiles_config_check_occurs_only_once(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     app = StaticFiles(directory=tmpdir)
     client = test_client_factory(app)
     assert not app.config_checked
@@ -162,7 +165,7 @@ def test_staticfiles_config_check_occurs_only_once(tmpdir, test_client_factory):
         client.get("/")
 
 
-def test_staticfiles_prevents_breaking_out_of_directory(tmpdir):
+def test_staticfiles_prevents_breaking_out_of_directory(tmpdir: Path) -> None:
     directory = os.path.join(tmpdir, "foo")
     os.mkdir(directory)
 
@@ -182,7 +185,7 @@ def test_staticfiles_prevents_breaking_out_of_directory(tmpdir):
     assert exc_info.value.detail == "Not Found"
 
 
-def test_staticfiles_never_read_file_for_head_method(tmpdir, test_client_factory):
+def test_staticfiles_never_read_file_for_head_method(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -195,7 +198,7 @@ def test_staticfiles_never_read_file_for_head_method(tmpdir, test_client_factory
     assert response.headers["content-length"] == "14"
 
 
-def test_staticfiles_304_with_etag_match(tmpdir, test_client_factory):
+def test_staticfiles_304_with_etag_match(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -208,15 +211,16 @@ def test_staticfiles_304_with_etag_match(tmpdir, test_client_factory):
     second_resp = client.get("/example.txt", headers={"if-none-match": last_etag})
     assert second_resp.status_code == 304
     assert second_resp.content == b""
+    second_resp = client.get("/example.txt", headers={"if-none-match": f'W/{last_etag}, "123"'})
+    assert second_resp.status_code == 304
+    assert second_resp.content == b""
 
 
 def test_staticfiles_304_with_last_modified_compare_last_req(
-    tmpdir, test_client_factory
-):
+    tmpdir: Path, test_client_factory: TestClientFactory
+) -> None:
     path = os.path.join(tmpdir, "example.txt")
-    file_last_modified_time = time.mktime(
-        time.strptime("2013-10-10 23:40:00", "%Y-%m-%d %H:%M:%S")
-    )
+    file_last_modified_time = time.mktime(time.strptime("2013-10-10 23:40:00", "%Y-%m-%d %H:%M:%S"))
     with open(path, "w") as file:
         file.write("<file content>")
     os.utime(path, (file_last_modified_time, file_last_modified_time))
@@ -224,20 +228,16 @@ def test_staticfiles_304_with_last_modified_compare_last_req(
     app = StaticFiles(directory=tmpdir)
     client = test_client_factory(app)
     # last modified less than last request, 304
-    response = client.get(
-        "/example.txt", headers={"If-Modified-Since": "Thu, 11 Oct 2013 15:30:19 GMT"}
-    )
+    response = client.get("/example.txt", headers={"If-Modified-Since": "Thu, 11 Oct 2013 15:30:19 GMT"})
     assert response.status_code == 304
     assert response.content == b""
     # last modified greater than last request, 200 with content
-    response = client.get(
-        "/example.txt", headers={"If-Modified-Since": "Thu, 20 Feb 2012 15:30:19 GMT"}
-    )
+    response = client.get("/example.txt", headers={"If-Modified-Since": "Thu, 20 Feb 2012 15:30:19 GMT"})
     assert response.status_code == 200
     assert response.content == b"<file content>"
 
 
-def test_staticfiles_html_normal(tmpdir, test_client_factory):
+def test_staticfiles_html_normal(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "404.html")
     with open(path, "w") as file:
         file.write("<h1>Custom not found page</h1>")
@@ -270,7 +270,7 @@ def test_staticfiles_html_normal(tmpdir, test_client_factory):
     assert response.text == "<h1>Custom not found page</h1>"
 
 
-def test_staticfiles_html_without_index(tmpdir, test_client_factory):
+def test_staticfiles_html_without_index(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "404.html")
     with open(path, "w") as file:
         file.write("<h1>Custom not found page</h1>")
@@ -295,7 +295,7 @@ def test_staticfiles_html_without_index(tmpdir, test_client_factory):
     assert response.text == "<h1>Custom not found page</h1>"
 
 
-def test_staticfiles_html_without_404(tmpdir, test_client_factory):
+def test_staticfiles_html_without_404(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "dir")
     os.mkdir(path)
     path = os.path.join(path, "index.html")
@@ -320,7 +320,7 @@ def test_staticfiles_html_without_404(tmpdir, test_client_factory):
     assert exc_info.value.status_code == 404
 
 
-def test_staticfiles_html_only_files(tmpdir, test_client_factory):
+def test_staticfiles_html_only_files(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "hello.html")
     with open(path, "w") as file:
         file.write("<h1>Hello</h1>")
@@ -338,8 +338,8 @@ def test_staticfiles_html_only_files(tmpdir, test_client_factory):
 
 
 def test_staticfiles_cache_invalidation_for_deleted_file_html_mode(
-    tmpdir, test_client_factory
-):
+    tmpdir: Path, test_client_factory: TestClientFactory
+) -> None:
     path_404 = os.path.join(tmpdir, "404.html")
     with open(path_404, "w") as file:
         file.write("<p>404 file</p>")
@@ -347,9 +347,7 @@ def test_staticfiles_cache_invalidation_for_deleted_file_html_mode(
     with open(path_some, "w") as file:
         file.write("<p>some file</p>")
 
-    common_modified_time = time.mktime(
-        time.strptime("2013-10-10 23:40:00", "%Y-%m-%d %H:%M:%S")
-    )
+    common_modified_time = time.mktime(time.strptime("2013-10-10 23:40:00", "%Y-%m-%d %H:%M:%S"))
     os.utime(path_404, (common_modified_time, common_modified_time))
     os.utime(path_some, (common_modified_time, common_modified_time))
 
@@ -377,8 +375,8 @@ def test_staticfiles_cache_invalidation_for_deleted_file_html_mode(
 
 
 def test_staticfiles_with_invalid_dir_permissions_returns_401(
-    tmp_path, test_client_factory
-):
+    tmp_path: Path, test_client_factory: TestClientFactory
+) -> None:
     (tmp_path / "example.txt").write_bytes(b"<file content>")
 
     original_mode = tmp_path.stat().st_mode
@@ -401,7 +399,7 @@ def test_staticfiles_with_invalid_dir_permissions_returns_401(
         tmp_path.chmod(original_mode)
 
 
-def test_staticfiles_with_missing_dir_returns_404(tmpdir, test_client_factory):
+def test_staticfiles_with_missing_dir_returns_404(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -415,7 +413,7 @@ def test_staticfiles_with_missing_dir_returns_404(tmpdir, test_client_factory):
     assert response.text == "Not Found"
 
 
-def test_staticfiles_access_file_as_dir_returns_404(tmpdir, test_client_factory):
+def test_staticfiles_access_file_as_dir_returns_404(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     path = os.path.join(tmpdir, "example.txt")
     with open(path, "w") as file:
         file.write("<file content>")
@@ -429,10 +427,23 @@ def test_staticfiles_access_file_as_dir_returns_404(tmpdir, test_client_factory)
     assert response.text == "Not Found"
 
 
+def test_staticfiles_filename_too_long(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
+    routes = [Mount("/", app=StaticFiles(directory=tmpdir), name="static")]
+    app = Starlette(routes=routes)
+    client = test_client_factory(app)
+
+    path_max_size = os.pathconf("/", "PC_PATH_MAX")
+    response = client.get(f"/{'a' * path_max_size}.txt")
+    assert response.status_code == 404
+    assert response.text == "Not Found"
+
+
 def test_staticfiles_unhandled_os_error_returns_500(
-    tmpdir, test_client_factory, monkeypatch
-):
-    def mock_timeout(*args, **kwargs):
+    tmpdir: Path,
+    test_client_factory: TestClientFactory,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
+    def mock_timeout(*args: typing.Any, **kwargs: typing.Any) -> None:
         raise TimeoutError
 
     path = os.path.join(tmpdir, "example.txt")
@@ -450,7 +461,7 @@ def test_staticfiles_unhandled_os_error_returns_500(
     assert response.text == "Internal Server Error"
 
 
-def test_staticfiles_follows_symlinks(tmpdir, test_client_factory):
+def test_staticfiles_follows_symlinks(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     statics_path = os.path.join(tmpdir, "statics")
     os.mkdir(statics_path)
 
@@ -471,7 +482,7 @@ def test_staticfiles_follows_symlinks(tmpdir, test_client_factory):
     assert response.text == "<h1>Hello</h1>"
 
 
-def test_staticfiles_follows_symlink_directories(tmpdir, test_client_factory):
+def test_staticfiles_follows_symlink_directories(tmpdir: Path, test_client_factory: TestClientFactory) -> None:
     statics_path = os.path.join(tmpdir, "statics")
     statics_html_path = os.path.join(statics_path, "html")
     os.mkdir(statics_path)
@@ -492,7 +503,7 @@ def test_staticfiles_follows_symlink_directories(tmpdir, test_client_factory):
     assert response.text == "<h1>Hello</h1>"
 
 
-def test_staticfiles_disallows_path_traversal_with_symlinks(tmpdir):
+def test_staticfiles_disallows_path_traversal_with_symlinks(tmpdir: Path) -> None:
     statics_path = os.path.join(tmpdir, "statics")
 
     root_source_path = tempfile.mkdtemp()
@@ -517,7 +528,7 @@ def test_staticfiles_disallows_path_traversal_with_symlinks(tmpdir):
     assert exc_info.value.detail == "Not Found"
 
 
-def test_staticfiles_avoids_path_traversal(tmp_path: Path):
+def test_staticfiles_avoids_path_traversal(tmp_path: Path) -> None:
     statics_path = tmp_path / "static"
     statics_disallow_path = tmp_path / "static_disallow"
 
